﻿using System;
using Bluetooth.Core.Services;
using System.Text;
using System.Threading.Tasks;
#if WINDOWS_PHONE_APP
using Windows.UI.ViewManagement;
using Windows.Phone.UI.Input;
#endif
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Input;
using Windows.UI.Xaml.Navigation;

// The Basic Page item template is documented at http://go.microsoft.com/fwlink/?LinkId=234237

namespace BluetoothDemo
{
    /// <summary>
    /// A basic page that provides characteristics common to most applications.
    /// </summary>
    public sealed partial class EnumerateBTDevices : Page
    {
#if WINDOWS_PHONE_APP
        StatusBar systemTray = null;
#endif

        public EnumerateBTDevices()
        {
            this.InitializeComponent();
#if WINDOWS_PHONE_APP
            this.Loaded += EnumerateBTDevices_Loaded;
            this.Unloaded += EnumerateBTDevices_Unloaded;
            systemTray = StatusBar.GetForCurrentView();
#endif
        }

#if WINDOWS_PHONE_APP
        void EnumerateBTDevices_Unloaded(object sender, Windows.UI.Xaml.RoutedEventArgs e)
        {
            HardwareButtons.BackPressed -= HardwareButtons_BackPressed;
        }

        void EnumerateBTDevices_Loaded(object sender, Windows.UI.Xaml.RoutedEventArgs e)
        {
            HardwareButtons.BackPressed += HardwareButtons_BackPressed;
        }

        void HardwareButtons_BackPressed(object sender, BackPressedEventArgs e)
        {
            e.Handled = true;
            if (this.Frame.CanGoBack)
            {
                this.Frame.GoBack();
            }
        }
#endif 
        protected async override void OnNavigatedTo(NavigationEventArgs e)
        {
            base.OnNavigatedTo(e);
            await EnumerateDevicesAsync();
        }

        private async void appbarbtnScan_Click(object sender, RoutedEventArgs e)
        {
            await EnumerateDevicesAsync();
        }

#if WINDOWS_APP
        private void backButton_Tapped(object sender, TappedRoutedEventArgs e)
        {
            if (this.Frame.CanGoBack)
            {
                this.Frame.GoBack();
            }
        }
#endif 

        public async Task EnumerateDevicesAsync()
        {
#if WINDOWS_APP
            prgrngSearchDevices.Visibility = Visibility.Visible;
            prgrngSearchDevices.IsActive = true;
#elif WINDOWS_PHONE_APP
            systemTray.ProgressIndicator.Text = "searching for devices";
            await systemTray.ProgressIndicator.ShowAsync();
#endif
            txtblkErrorBtDevices.Text = "Searching for devices . . .";
            txtblkErrorBtDevices.Visibility = Visibility.Visible;
            lstvwBtDevices.Visibility = Visibility.Collapsed;

            BluetoothService btService = BluetoothService.GetDefault();
            btService.SearchForPairedDevicesFailed += btService_SearchForPairedDevicesFailed;
            btService.SearchForPairedDevicesSucceeded += btService_SearchForPairedDevicesSucceeded;
            await btService.SearchForPairedDevicesAsync();
        }

#if WINDOWS_APP
        void btService_SearchForPairedDevicesSucceeded(object sender, SearchForPairedDevicesSucceededEventArgs e)
#elif WINDOWS_PHONE_APP
        async void btService_SearchForPairedDevicesSucceeded(object sender, SearchForPairedDevicesSucceededEventArgs e)
#endif
        {
            (sender as BluetoothService).SearchForPairedDevicesFailed -= btService_SearchForPairedDevicesFailed;
            (sender as BluetoothService).SearchForPairedDevicesSucceeded -= btService_SearchForPairedDevicesSucceeded;

#if WINDOWS_APP
            prgrngSearchDevices.Visibility = Visibility.Collapsed;
            prgrngSearchDevices.IsActive = false;
#elif WINDOWS_PHONE_APP
            await systemTray.ProgressIndicator.HideAsync();
#endif

            txtblkErrorBtDevices.Visibility = Visibility.Collapsed;
            lstvwBtDevices.Visibility = Visibility.Visible;
            this.cvBtDevices.Source = e.PairedDevices;
        }

#if WINDOWS_APP
        void btService_SearchForPairedDevicesFailed(object sender, SearchForPairedDevicesFailedEventArgs e)
#elif WINDOWS_PHONE_APP
        async void btService_SearchForPairedDevicesFailed(object sender, SearchForPairedDevicesFailedEventArgs e)
#endif
        {
            (sender as BluetoothService).SearchForPairedDevicesFailed -= btService_SearchForPairedDevicesFailed;
            (sender as BluetoothService).SearchForPairedDevicesSucceeded -= btService_SearchForPairedDevicesSucceeded;

#if WINDOWS_APP
            prgrngSearchDevices.Visibility = Visibility.Collapsed;
            prgrngSearchDevices.IsActive = false;
#elif WINDOWS_PHONE_APP
            await systemTray.ProgressIndicator.HideAsync();
#endif

            txtblkErrorBtDevices.Visibility = Visibility.Visible;
            lstvwBtDevices.Visibility = Visibility.Collapsed;
            txtblkErrorBtDevices.Text = "No Devices could be found. Ensure Bluetooth is switched on and atleast one paired device is available";
        }

        private void txtblkDeviceName_Tapped(object sender, TappedRoutedEventArgs e)
        {
            this.Frame.Navigate(typeof(StartSharing), (sender as TextBlock).DataContext as BluetoothDevice);
        }

        private async void appbarbtnHelp_Click(object sender, RoutedEventArgs e)
        {
            StringBuilder builder = new StringBuilder();
#if WINDOWS_APP
            builder.AppendLine("Use 'BluetoothService.ScanForPairedDevicesAsync()' method to scan for all paired devices with this device.");
            builder.AppendLine();
            builder.AppendLine("The system will ask you to 'allow' or 'block' the use. You must click/tap on 'allow' for the enumerated paired devices. If you click/tap 'block' then that will be permanently blocked from being enumerated.");
            builder.AppendLine();
            builder.AppendLine("Handle 'SearchForPairedDevicesSucceeded' event to get the list of paired devices.");
            builder.AppendLine();
            builder.AppendLine("Handle 'SearchForPairedDevicesFailed' for failure enumeration.");
#elif WINDOWS_PHONE_APP
            builder.AppendLine("Use 'BluetoothService.ScanForPairedDevicesAsync()' method to scan for all paired devices with this device.");
            builder.AppendLine();
            builder.AppendLine("Handle 'SearchForPairedDevicesSucceeded' event to get the list of paired devices.");
            builder.AppendLine();
            builder.AppendLine("Handle 'SearchForPairedDevicesFailed' for failure enumeration.");
#endif

            await (App.Current as App).ShowHelpAsync(builder.ToString());
        }
    }
}
