﻿using Bluetooth.Core.Services;
using Bluetooth.Services.Obex;
using System;
using System.Collections.ObjectModel;
using System.IO;
using System.Linq;
using Windows.ApplicationModel;
#if WINDOWS_PHONE_APP
using Windows.Phone.UI.Input;
#endif
using Windows.Storage;
using Windows.UI.Core;
using Windows.UI.Popups;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Navigation;
using System.Text;

// The Basic Page item template is documented at http://go.microsoft.com/fwlink/?LinkId=234237

namespace BluetoothDemo
{
    /// <summary>
    /// A basic page that provides characteristics common to most applications.
    /// </summary>
    public sealed partial class StartSharing : Page
    {
        ObexService obexService = null;
        BluetoothDevice BtDevice = null;
        ObservableCollection<FileItemToShare> filesToShare = null;

        public StartSharing()
        {
            this.InitializeComponent();
#if WINDOWS_PHONE_APP
            this.Loaded += StartSharing_Loaded;
            this.Unloaded += StartSharing_Unloaded;
#endif
        }

#if WINDOWS_PHONE_APP
        void StartSharing_Unloaded(object sender, Windows.UI.Xaml.RoutedEventArgs e)
        {
            HardwareButtons.BackPressed -= HardwareButtons_BackPressed;
        }

        void StartSharing_Loaded(object sender, Windows.UI.Xaml.RoutedEventArgs e)
        {
            HardwareButtons.BackPressed += HardwareButtons_BackPressed;
        }

        async void HardwareButtons_BackPressed(object sender, BackPressedEventArgs e)
        {
            e.Handled = true;
            if (this.Frame.CanGoBack)
            {
                if (obexService != null)
                {
                    await obexService.AbortAsync();
                }
                this.Frame.GoBack();
            }
        }
#endif

        async protected override void OnNavigatedTo(NavigationEventArgs e)
        {
            base.OnNavigatedTo(e);

            if (e.Parameter == null || !(e.Parameter is BluetoothDevice))
            {
                MessageDialog messageBox = new MessageDialog("Invalid navigation detected. Moving to Main Page", "Bluetooth Hub");
                messageBox.Commands.Add(new UICommand("OK", (uiCommand) =>
                {
                    this.Frame.Navigate(typeof(MainPage));
                }));
                await messageBox.ShowAsync();
                return;
            }

            BtDevice = e.Parameter as BluetoothDevice;
            StorageFolder installedFolder = Package.Current.InstalledLocation;
            filesToShare = new ObservableCollection<FileItemToShare>(from IStorageFile musicFile in await (await installedFolder.GetFolderAsync("DemoData")).GetFilesAsync()
                                                                     where Path.GetExtension(musicFile.Path).Equals(".zip")
                                                                     select new FileItemToShare(Path.GetFileNameWithoutExtension(musicFile.Name), Path.GetDirectoryName(musicFile.Path), musicFile));
            this.cvFileItems.Source = filesToShare;

            obexService = ObexService.GetDefaultForBluetoothDevice(BtDevice);
            obexService.DeviceConnected += obexService_DeviceConnected;
            obexService.ServiceConnected += obexService_ServiceConnected;
            obexService.ConnectionFailed += obexService_ConnectionFailed;
            obexService.DataTransferProgressed += obexService_DataTransferProgressed;
            obexService.DataTransferSucceeded += obexService_DataTransferSucceeded;
            obexService.DataTransferFailed += obexService_DataTransferFailed;
            obexService.Disconnecting += obexService_Disconnecting;
            obexService.Disconnected += obexService_Disconnected;
            obexService.Aborted += obexService_Aborted;
            await obexService.ConnectAsync();
        }

        async void obexService_DataTransferFailed(object sender, DataTransferFailedEventArgs e)
        {
            await this.Dispatcher.RunAsync(CoreDispatcherPriority.Normal, () =>
            {
                System.Diagnostics.Debug.WriteLine(e.ExceptionObject.ToString());
                this.filesToShare[0].ShareStatus = FileShareStatus.Error;
                this.filesToShare[0].Progress = 0;
                FileItemToShare fileToShare = this.filesToShare[0];
                this.filesToShare.RemoveAt(0);
                this.filesToShare.Add(fileToShare);
            });
        }

        async void obexService_Aborted(object sender, EventArgs e)
        {
            System.Diagnostics.Debug.WriteLine("aborted transfer");
            await this.Dispatcher.RunAsync(CoreDispatcherPriority.Normal, () =>
            {
                if (!filesToShare.Count.Equals(0))
                {
                    filesToShare.RemoveAt(0);
                }
            });
        }

        async void obexService_Disconnected(object sender, EventArgs e)
        {
            System.Diagnostics.Debug.WriteLine("disconnected from service and device");
            await this.Dispatcher.RunAsync(CoreDispatcherPriority.Normal, async () =>
            {
                obexService.Aborted -= obexService_Aborted;
                obexService.ConnectionFailed -= obexService_ConnectionFailed;
                obexService.DataTransferFailed -= obexService_DataTransferFailed;
                obexService.DataTransferProgressed -= obexService_DataTransferProgressed;
                obexService.DataTransferSucceeded -= obexService_DataTransferSucceeded;
                obexService.DeviceConnected -= obexService_DeviceConnected;
                obexService.Disconnected -= obexService_Disconnected;
                obexService.Disconnecting -= obexService_Disconnecting;
                obexService.ServiceConnected -= obexService_ServiceConnected;
                obexService = null;

                if (this.filesToShare.Count.Equals(0))
                {
                    MessageDialog messageBox = new MessageDialog("All files shared successfully", "Bluetooth OBEX DemoApp");
                    messageBox.Commands.Add(new UICommand("OK", (uiCommand) =>
                    {
                        this.Frame.Navigate(typeof(MainPage));
                    }));
                    await messageBox.ShowAsync();
                }
                else if (this.filesToShare[0].ShareStatus.Equals(FileShareStatus.Error) || this.filesToShare[0].ShareStatus.Equals(FileShareStatus.Cancelled))
                {
                    MessageDialog messageBox = new MessageDialog("Some files could not be shared successfully", "Bluetooth OBEX DemoApp");
                    messageBox.Commands.Add(new UICommand("OK", (uiCommand) =>
                    {
                        this.Frame.Navigate(typeof(MainPage));
                    }));
                    await messageBox.ShowAsync();
                }
                else
                {
                    obexService = ObexService.GetDefaultForBluetoothDevice(BtDevice);
                    obexService.DeviceConnected += obexService_DeviceConnected;
                    obexService.ServiceConnected += obexService_ServiceConnected;
                    obexService.ConnectionFailed += obexService_ConnectionFailed;
                    obexService.DataTransferProgressed += obexService_DataTransferProgressed;
                    obexService.DataTransferSucceeded += obexService_DataTransferSucceeded;
                    obexService.DataTransferFailed += obexService_DataTransferFailed;
                    obexService.Disconnecting += obexService_Disconnecting;
                    obexService.Disconnected += obexService_Disconnected;
                    obexService.Aborted += obexService_Aborted;
                    await obexService.ConnectAsync();
                }
            });
        }

        async void obexService_Disconnecting(object sender, EventArgs e)
        {
            await this.Dispatcher.RunAsync(CoreDispatcherPriority.Normal, () =>
            {
                System.Diagnostics.Debug.WriteLine("disconnecting from remote device");
            });
        }

        async void obexService_DataTransferSucceeded(object sender, EventArgs e)
        {
            await this.Dispatcher.RunAsync(CoreDispatcherPriority.Normal, () =>
            {
                System.Diagnostics.Debug.WriteLine("Data transfer of current file succeeded");
                this.filesToShare.RemoveAt(0);
            });
        }

        async void obexService_DataTransferProgressed(object sender, DataTransferProgressedEventArgs e)
        {
            System.Diagnostics.Debug.WriteLine("{0} bytes sent, {1}% transfer progressed", e.TransferInBytes, e.TransferInPercentage * 100);
            await this.Dispatcher.RunAsync(CoreDispatcherPriority.Normal, () =>
            {
                FileItemToShare fileToShare = this.filesToShare[0];
                fileToShare.Progress = e.TransferInBytes;

            });
        }

        async void obexService_ConnectionFailed(object sender, ConnectionFailedEventArgs e)
        {
            await this.Dispatcher.RunAsync(CoreDispatcherPriority.Normal, () =>
            {
                System.Diagnostics.Debug.WriteLine("Connection to obex service on target device failed");
                FileItemToShare fileToShare = this.filesToShare[0];
                fileToShare.ShareStatus = FileShareStatus.Error;
                fileToShare.Progress = 0;
                this.filesToShare.RemoveAt(0);
                this.filesToShare.Add(fileToShare);
            });
        }

        async void obexService_ServiceConnected(object sender, EventArgs e)
        {
            await this.Dispatcher.RunAsync(CoreDispatcherPriority.Normal, () =>
            {
                System.Diagnostics.Debug.WriteLine("Device connected to remote obex service on target device");
                this.filesToShare[0].ShareStatus = FileShareStatus.Sharing;
            });
        }

        async void obexService_DeviceConnected(object sender, EventArgs e)
        {
            await this.Dispatcher.RunAsync(CoreDispatcherPriority.Normal, async () =>
            {
                System.Diagnostics.Debug.WriteLine("Device connected to remote socket");
                if (this.filesToShare.Count > 0)
                {
                    this.filesToShare[0].ShareStatus = FileShareStatus.Connecting;
                    await obexService.SendFileAsync(this.filesToShare[0].FileToShare);
                }
                else
                {
                    obexService.Aborted -= obexService_Aborted;
                    obexService.ConnectionFailed -= obexService_ConnectionFailed;
                    obexService.DataTransferFailed -= obexService_DataTransferFailed;
                    obexService.DataTransferProgressed -= obexService_DataTransferProgressed;
                    obexService.DataTransferSucceeded -= obexService_DataTransferSucceeded;
                    obexService.DeviceConnected -= obexService_DeviceConnected;
                    obexService.Disconnected -= obexService_Disconnected;
                    obexService.Disconnecting -= obexService_Disconnecting;
                    obexService.ServiceConnected -= obexService_ServiceConnected;
                    obexService = null;
                }
            });
        }

#if WINDOWS_APP
        async private void backButton_Click(object sender, Windows.UI.Xaml.RoutedEventArgs e)
        {
            if (this.Frame.CanGoBack)
            {
                if (obexService != null)
                {
                    await obexService.AbortAsync();
                }
                this.Frame.GoBack();
            }
        }
#endif

        async private void appbarbtnHelp_Click(object sender, Windows.UI.Xaml.RoutedEventArgs e)
        {
            StringBuilder messageBuilder = new StringBuilder();
            messageBuilder.AppendLine("Use ObexService.GetDefaultForBluetoothDevice(BluetoothDevice) to get the instance of OBEX Service");
            messageBuilder.AppendLine();
            messageBuilder.AppendLine("Handle 'DeviceConnected' event to get notified when the device connects to target device over Bluetooth Socket");
            messageBuilder.AppendLine();
            messageBuilder.AppendLine("Handle 'ConnectionFailed' event to get notified if the connection to target device fails");
            messageBuilder.AppendLine();
            messageBuilder.AppendLine("Handle 'ServiceConnected' event to get notified when the device connects to OBEX server running on target device");
            messageBuilder.AppendLine();
            messageBuilder.AppendLine("Handle 'DataTransferProgressed' event to get notified about the bytes and percentage of file shared");
            messageBuilder.AppendLine();
            messageBuilder.AppendLine("Handle 'DataTransferSucceeded' event to get notified the file sharing is completed with success");
            messageBuilder.AppendLine();
            messageBuilder.AppendLine("Handle 'DataTransferFailed' event to get notified about the failure in file sharing");
            messageBuilder.AppendLine();
            messageBuilder.AppendLine("Handle 'Disconnecting' event to get notified about state of disconnecting from Obex Server and target device");
            messageBuilder.AppendLine();
            messageBuilder.AppendLine("Handle 'Disconnected' event to get notified about complete disconnect from Obex Server and target device");
            messageBuilder.AppendLine();
            messageBuilder.AppendLine("Handle 'Aborted' event to get notified when the file sharing is aborted");

            await (App.Current as App).ShowHelpAsync(messageBuilder.ToString());
        }
    }
}